local addon, ns = ...

local cfg = ns.cfg
local lib = ns.lib

-- Unit has an Aura
function hasUnitAura(unit, name)

local _, _, _, count, _, _, _, caster = UnitAura(unit, name)
if (caster and caster == "player") then
return count
end
end

-- Unit has a Debuff
function hasUnitDebuff(unit, name)

local _, _, _, count, _, _, _, _ = UnitDebuff(unit, name)
if (count) then return count
end
end

local MyPvPUpdate = function(self, event, unit)
if(unit ~= self.unit) then return end

local pvp = self.MyPvP
if(pvp) then
local factionGroup = UnitFactionGroup(unit)
-- FFA!
if(UnitIsPVPFreeForAll(unit)) then
pvp:SetTexture([[Interface\TargetingFrame\UI-PVP-FFA]])
pvp:Show()
elseif(UnitIsPVP(unit) and factionGroup) then
if(factionGroup == 'Horde') then
pvp:SetTexture([[Interface\Addons\oUF_sqw\media\Horde]])
else
pvp:SetTexture([[Interface\Addons\oUF_sqw\media\Alliance]])
end
pvp:Show()
else
pvp:Hide()
end
end
end

oUF.colors.smooth = {0.08, 0.08, 0.08, 0.08, 0.08, 0.08, 0.08, 0.08, 0.08,}

-----------------------------
-- STYLE FUNCTIONS
-----------------------------
local backdrop = {
bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=],
insets = {top = 0, left = 0, bottom = 0, right = 0},
}

local backdrop2 = {
bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=],
insets = {top = -1, left = -1, bottom = -1, right = -1},
}

local frameBD = {
edgeFile = glowTex, edgeSize = 5,
bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=],
insets = {left = 3, right = 3, top = 3, bottom = 3}
}
local createBackdrop = function(parent, anchor) 
local frame = CreateFrame("Frame", nil, parent)
frame:SetFrameStrata("LOW")

if pixelborder then
frame:SetAllPoints(anchor)
frame:SetBackdrop(backdrop2)
else
frame:SetPoint("TOPLEFT", anchor, "TOPLEFT", -4, 4)
frame:SetPoint("BOTTOMRIGHT", anchor, "BOTTOMRIGHT", 4, -4)
frame:SetBackdrop(frameBD)
end

frame:SetBackdropColor(.05, .05, .05, 0.2)
frame:SetBackdropBorderColor(0, 0, 0, 1)
return frame
end
ns.backdrop = createBackdrop

local UnitSpecific = {


-- Player

player = function(self, ...)

self.mystyle = "player"

-- Size and Scale
self:SetScale(cfg.scale)
self:SetSize(250, 29)

-- Portrait
if cfg.showportraitplayer then
self.Portrait = CreateFrame("PlayerModel", nil, self)
self.Portrait:SetWidth(cfg.playerportraitwidth)
self.Portrait:SetHeight(cfg.playerportraitheight)
self.Portrait:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", -7, 4)
--self.PortraitBackdrop = createBackdrop(self, self.Portrait)
end

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
lib.gen_ppbar(self)
lib.gen_RaidMark(self)
lib.gen_combat_feedback(self)
lib.gen_floating_combat_feedback(self)
lib.gen_InfoIcons(self)
lib.HealPred(self)

-- Buffs and Debuffs
if cfg.showPlayerAuras then
BuffFrame:Hide()
lib.createBuffs(self)
lib.createDebuffs(self)
lib.gen_WeaponEnchant(self)
end

self.Health.frequentUpdates = true
self.Health.colorSmooth = true
self.Health.Smooth = false
-- self.Health.bg.multiplier = 0.2
self.Power.colorPower = true
self.Power.Smooth = true
self.Power.frequentUpdates = true
self.Power.bg.multiplier = 0.2
lib.gen_exp(self)
lib.gen_rep(self)
lib.gen_castbar(self)
lib.debuffHighlight(self)
--	self.Power.PostUpdate = lib.setPowerArrowColor

-- PvP Icon
local pvp = self.Health:CreateTexture(nil, "OVERLAY")
pvp:SetHeight(32)
pvp:SetWidth(32)
--pvp:SetPoint("TOPRIGHT", 40, 10)
pvp:SetPoint("TOPLEFT", -25, 25)
self.MyPvP = pvp

-- This makes oUF update the information.
self:RegisterEvent("UNIT_FACTION", MyPvPUpdate)
-- This makes oUF update the information on forced updates.
table.insert(self.__elements, MyPvPUpdate)

if cfg.showRunebar then lib.genRunes(self) end
--if cfg.showEclipsebar then lib.addEclipseBar(self) end
--if cfg.showClassIcons then lib.gen_ClassIcons(self) end
--if cfg.showShards then lib.genShards(self) end
if cfg.showClassbar then lib.gen_Classbar(self) end
lib.RogueComboPoints(self)
-- Addons
--if cfg.showTotemBar then lib.gen_TotemBar(self) end

-- Fader
self.FadeMinAlpha = 0
self.FadeInSmooth = 0.4
self.FadeOutSmooth = 1.5
self.FadeCasting = true
self.FadeCombat = true
self.FadeTarget = true
self.FadeHealth = true
self.FadePower = true
self.FadeHover = true

end,

-- Target
target = function(self, ...)

self.mystyle = "target"

-- Size and Scale
self:SetScale(cfg.scale)
self:SetSize(250,29)

-- Portrait
if cfg.showportraittarget then
self.Portrait = CreateFrame("PlayerModel", nil, self)

self.Portrait:SetWidth(cfg.targetportraitwidth)
self.Portrait:SetHeight(cfg.targetportraitheight)
self.Portrait:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", -7, 4)
--self.PortraitBackdrop = createBackdrop(self, self.Portrait)
end

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--lib.gen_ppbar(self)
lib.gen_RaidMark(self)
lib.gen_combat_feedback(self)
lib.gen_floating_combat_feedback(self)
--lib.RogueComboPoints(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
self.Health.Smooth = true
lib.gen_castbar(self)
--lib.gen_mirrorcb(self)
lib.debuffHighlight(self)
lib.HealPred(self)

-- PvP Icon
local pvp = self.Health:CreateTexture(nil, "OVERLAY")
--pvp:SetTexture(1, 0, 0)
pvp:SetHeight(36)
pvp:SetWidth(36)
pvp:SetPoint("TOPLEFT", -25, 25)
--pvp:SetPoint("TOPRIGHT", 45, 15)
self.MyPvP = pvp

-- This makes oUF update the information.
self:RegisterEvent("UNIT_FACTION", MyPvPUpdate)
table.insert(self.__elements, MyPvPUpdate)

if cfg.showTargetBuffs then	lib.createBuffs(self) end
if cfg.showTargetDebuffs then lib.createDebuffs(self) end
end,

-- Target of Target
targettarget = function(self, ...)

self.mystyle = "tot"

-- Size and Scale
self:SetScale(cfg.scale)
self:SetSize(90, 29)

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--lib.gen_ppbar(self)
lib.gen_RaidMark(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
self.Health.Smooth = true
end,

-- Focus
focus = function(self, ...)

self.mystyle = "focus"

-- Size and Scale
self:SetScale(cfg.scale)
self:SetSize(90,29)

-- Portrait
if cfg.portraits then
self.Portrait = CreateFrame("PlayerModel", nil, self)
self.Portrait:SetWidth(60)
self.Portrait:SetHeight(40)
self.Portrait:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", -4, -20)
self.PortraitBackdrop = createBackdrop(self, self.Portrait)
end

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--lib.gen_ppbar(self)
lib.gen_RaidMark(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.Smooth = true
self.Health.colorSmooth = true

end,

-- Focus target
focustarget = function(self, ...)

self.mystyle = "focustarget"

-- Size and Scale
self:SetScale(cfg.scale)
self:SetSize(90,29)

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--lib.gen_ppbar(self)
lib.gen_RaidMark(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true

end}



-- Pet style
local function CreatePetStyle(self, unit)

self.mystyle = "pet"

-- Size and Scale
self:SetScale(cfg.scale)
--self:SetSize(60,40)
self:SetSize(90,29)

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--	lib.gen_ppbar(self)
lib.gen_RaidMark(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
self.Health.Smooth = true

-- Fader
local unit = "pet"
self.FadeMinAlpha = 0
self.FadeInSmooth = 0.4
self.FadeOutSmooth = 1.5
self.FadeCasting = true
self.FadeCombat = true
self.FadeTarget = true
self.FadeHealth = true
self.FadePower = true
self.FadeHover = true

end

-- Partypet style
local function CreatePartyPetStyle(self)

-- Size and Scale
--self:SetScale(cfg.scale)
self:SetSize(65,40)
self.mystyle = "partypet"
self.Range = {
insideAlpha = 1,
outsideAlpha = .3,
}

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--lib.gen_ppbar(self)
lib.gen_RaidMark(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
self.Health.Smooth = true

end

-- Party style
local function CreatePartyStyle(self)
self.menu = lib.menu
self:RegisterForClicks("AnyUp")
self:SetAttribute("*type2", "menu")
self:SetScript("OnEnter", UnitFrame_OnEnter)
self:SetScript("OnLeave", UnitFrame_OnLeave)
if self:GetAttribute("unitsuffix") == "pet" then
return CreatePartyPetStyle(self)
end
self.mystyle = "party"
self.Range = {
insideAlpha = 1,
outsideAlpha = .3,
}

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
lib.gen_ppbar(self)
lib.gen_RaidMark(self)
lib.ReadyCheckIndicator(self)
lib.gen_GroupRoleIndicator(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
self.Health.Smooth = true
self.Power.Smooth = true
-- self.Health.bg.multiplier = 0.3
self.Power.colorClass = true
self.Power.bg.multiplier = 0.5

lib.gen_InfoIcons(self)
lib.CreateTargetBorder(self)
lib.CreateThreatIndicatorBorder(self)
lib.HealPred(self)
lib.debuffHighlight(self)
lib.raidDebuffs(self)

-- Portraits (Party)
if cfg.showportraitparty then
self.Portrait = CreateFrame("PlayerModel", nil, self)
self.Portrait:SetWidth(cfg.partyportraitwidth)
self.Portrait:SetHeight(cfg.partyportraitheight)
self.Portrait:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", -8, -2)
self.PortraitBackdrop = createBackdrop(self, self.Portrait)
end

-- Position and size
local GroupRoleIndicator = self:CreateTexture(nil, "OVERLAY")
GroupRoleIndicator:SetSize(16, 16)
GroupRoleIndicator:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, 0)

-- Register it with oUF
self.GroupRoleIndicator = GroupRoleIndicator
self.Health.PostUpdate = lib.PostUpdateRaidFrame
self:RegisterEvent('PLAYER_TARGET_CHANGED', lib.ChangedTarget)
self:RegisterEvent('RAID_ROSTER_UPDATE', lib.ChangedTarget)
self:RegisterEvent("UNIT_THREAT_LIST_UPDATE", lib.UpdateThreat)
self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE", lib.UpdateThreat)
end

-- Raid frames layout
local CreateRaidStyle = function(self, unit, isSingle)
self.menu = lib.menu
self:RegisterForClicks("AnyUp")
self:SetAttribute("*type2", "menu")
self:SetScript("OnEnter", UnitFrame_OnEnter)
self:SetScript("OnLeave", UnitFrame_OnLeave)
self.mystyle = "sqwRaid"
self.Range = {
insideAlpha = 1,
outsideAlpha = .3,
}

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--lib.gen_ppbar(self)
lib.gen_RaidMark(self)
lib.ReadyCheckIndicator(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
--self.Health.Smooth = true
--self.Power.Smooth = true
-- self.Health.bg.multiplier = 0.3
--self.Power.colorClass = true
--self.Power.bg.multiplier = 0.5
lib.gen_InfoIcons(self)
lib.CreateTargetBorder(self)
lib.CreateThreatIndicatorBorder(self)
lib.HealPred(self)
lib.debuffHighlight(self)
lib.raidDebuffs(self)
lib.createAuraWatch(self, unit)

self.Health.PostUpdate = lib.PostUpdateRaidFrame
self:RegisterEvent('PLAYER_TARGET_CHANGED', lib.ChangedTarget)
self:RegisterEvent('GROUP_ROSTER_UPDATE', lib.ChangedTarget)
self:RegisterEvent("UNIT_THREAT_LIST_UPDATE", lib.UpdateThreat)
self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE", lib.UpdateThreat)
end


-- The Shared Style Function
local GlobalStyle = function(self, unit, isSingle)

self.menu = lib.spawnMenu
self:RegisterForClicks('AnyDown')

-- Call Unit Specific Styles
if(UnitSpecific[unit]) then
return UnitSpecific[unit](self)
end
end

-- The Shared Style Function for Party and Raid
local GroupGlobalStyle = function(self, unit)

self.menu = lib.spawnMenu
self:RegisterForClicks('AnyDown')

-- Call Unit Specific Styles
if(UnitSpecific[unit]) then
return UnitSpecific[unit](self)
end
end

-- Boss frames layout
local function CreateBossStyle(self, unit)

self.mystyle = "sqwBoss"

self.Range = {
insideAlpha = 1,
outsideAlpha = .3,
}

-- Size and Scale
self:SetSize(180,29)

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
--lib.gen_ppbar(self)
lib.gen_RaidMark(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
lib.gen_castbar(self)
lib.gen_mirrorcb(self)

end

local function CreateMTStyle(self,unit,isSingle)
self.mystyle = "oUF_MT"
self:SetSize(150,29)

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
lib.gen_ppbar(self)
lib.gen_RaidMark(self)
lib.ReadyCheckIndicator(self)
lib.createDebuffs(self)
self.Health.frequentUpdates = false
self.Health.colorClass = true

end 

local function CreateArenaStyle(self, unit, isSingle)
self.mystyle = "sqwArena"
self.Range = {
insideAlpha = 1,
outsideAlpha = .3,
}

-- Size and Scale
self:SetSize(150,29)

-- Generate Bars
lib.gen_hpbar(self)
lib.gen_hpstrings(self)
lib.gen_highlight(self)
lib.gen_ppbar(self)
lib.gen_RaidMark(self)

--style specific stuff
self.Health.frequentUpdates = true
self.Health.colorSmooth = true
--self.Health.Smooth = true
self.Power.frequentUpdates = true
self.Power.Smooth = true
self.Power.colorHealth = true
self.Power.colorClass = true
self.Power.bg.multiplier = 0.5
lib.gen_castbar(self)
lib.gen_mirrorcb(self)

lib.createBuffs(self) 
lib.createDebuffs(self) 

end

-----------------------------
-- SPAWN UNITS
-----------------------------

oUF:RegisterStyle('sqw', GlobalStyle)
oUF:RegisterStyle('sqwPet', CreatePetStyle)
oUF:RegisterStyle('sqwParty', CreatePartyStyle)
oUF:RegisterStyle('sqwRaid', CreateRaidStyle)
oUF:RegisterStyle('sqwMT', CreateMTStyle)
oUF:RegisterStyle('sqwArena', CreateArenaStyle)
oUF:RegisterStyle('sqwBoss', CreateBossStyle)

oUF:Factory(function(self)
-- Single Frames
self:SetActiveStyle('sqw')
self:Spawn('player'):SetPoint("CENTER", UIParent, cfg.PlayerRelativePoint, cfg.PlayerX, cfg.PlayerY)
self:Spawn('target'):SetPoint("CENTER", UIParent, cfg.TargetRelativePoint, cfg.TargetX, cfg.TargetY)
if cfg.showtot then self:Spawn('targettarget'):SetPoint("TOPLEFT",oUF_sqwTarget,"TOPRIGHT", 60, 0) end
--	if cfg.showpet then self:Spawn('pet'):SetPoint("TOPRIGHT",oUF_sqwPlayer,"TOPLEFT", -70, 0) end
if cfg.showfocus then self:Spawn('focus'):SetPoint("BOTTOMRIGHT", oUF_sqwTarget, cfg.FocusRelativePoint, cfg.FocusX, cfg.FocusY) end
if cfg.showfocustarget then self:Spawn('focustarget'):SetPoint("BOTTOMLEFT",oUF_sqwFocus,"TOPLEFT", 0, -100) end

-- Pet Frames
if cfg.showpet then 
self:SetActiveStyle("sqwPet")
local pet = self:Spawn("pet", "oUF_sqwPetFrame")
pet:SetPoint("TOPRIGHT",oUF_sqwPlayer,"TOPLEFT", -80, 0)

pet:SetScale(cfg.scale)

end

-- Party Frames
if cfg.ShowParty then
self:SetActiveStyle('sqwParty')
local party = oUF:SpawnHeader('sqwParty', nil, 'custom  [group:party,nogroup:raid][@raid6,noexists,group:raid] show;hide',

"showParty", true,
"showPlayer", true,
'template','oUF_sqwPartyPet',
--'useOwnerUnit', true,
"yoffset", -30,
'groupBy', 'ASSIGNEDROLE',
'groupingOrder', 'TANK,HEALER,DAMAGER',

"oUF-initialConfigFunction", ([[
self:SetWidth(%d)
self:SetHeight(%d)
]]):format(140, 32))
party:SetScale(cfg.partyScale)
party:SetPoint('BOTTOM', UIParent, 'CENTER', cfg.PartyX, cfg.PartyY)
else
oUF:DisableBlizzard'party'
end	

-- Raid Frames
if cfg.ShowRaid then
self:SetActiveStyle('sqwRaid')

local raid25 = oUF:SpawnHeader("oUF_Raid", nil, "custom show; [@raid2,exists] show; hide", -- Raid frames for 6-25 players.
"showRaid", cfg.ShowRaid,
"showSolo", false,
"showPlayer", true,
"showParty", false,
"xoffset", 17,
"yOffset", -17,--raid vertical -?? increases gap

"groupFilter", "1,2,3,4,5",
"groupBy", "GROUP",
"groupingOrder",      "1,2,3,4,5,6",
"sortMethod", "INDEX",
"maxColumns", 5,
"unitsPerColumn", 5,
"columnSpacing", 9,
"point", "TOP",
"startingIndex",		1,
"columnAnchorPoint", "LEFT",
"oUF-initialConfigFunction", ([[
self:SetWidth(%d)
self:SetHeight(%d)
]]):format(65, 16))
--raid25:SetScale(cfg.raidScale)
--raid25:SetPoint('TOPLEFT', UIParent, 'TOPLEFT', cfg.RaidX, cfg.RaidY)
raid25:SetPoint('CENTER', UIParent, 'CENTER', cfg.RaidX+200, cfg.RaidY+100)

end 

-- Main Tank Frames
if cfg.showMTFrames then
self:SetActiveStyle('sqwMT')
local tank = oUF:SpawnHeader('oUF_MT', nil, 'raid',
'oUF-initialConfigFunction', ([[
self:SetWidth(%d)
self:SetHeight(%d)
]]):format(80, 22),
'showRaid', true,
'groupFilter', 'MAINTANK',
'yOffset', 8,
'point' , 'BOTTOM',
'template', 'oUF_MainTank')
tank:SetPoint("TOP", UIParent, "TOP", cfg.TankX, cfg.TankY)
end

-- Boss Frames
if cfg.showBossFrames then
self:SetActiveStyle('sqwBoss')
local boss = {}
for i = 1, MAX_BOSS_FRAMES do
boss[i] = self:Spawn("boss"..i, "oUF_Boss"..i)
if i == 1 then
boss[i]:SetPoint("CENTER", UIParent, "CENTER", cfg.BossX, cfg.BossY)
else
boss[i]:SetPoint("BOTTOMRIGHT", boss[i-1], "BOTTOMRIGHT", 0, 90)
end
end
end

-- Arena Frames
if cfg.showArenaFrames then
self:SetActiveStyle('sqwArena')

local arena = {}
for i = 1, 5 do
arena[i] = self:Spawn("arena"..i, "oUF_Arena"..i)
if i == 1 then
arena[i]:SetPoint("BOTTOMRIGHT", UIParent, "TOPRIGHT", cfg.BossX, cfg.BossY)
else
arena[i]:SetPoint("BOTTOMRIGHT", arena[i-1], "BOTTOMRIGHT", 0, 90)
end
arena[i]:SetSize(150, 30)
end	

local sqwPrepArena = {}
for i = 1, 5 do
sqwPrepArena[i] = CreateFrame("Frame", "sqwPrepArena"..i, UIParent)
sqwPrepArena[i]:SetAllPoints(arena[i])
sqwPrepArena[i]:SetBackdropColor(0,0,0)
--CreateShadow(sqwPrepArena[i])
sqwPrepArena[i].Health = CreateFrame("StatusBar", nil, sqwPrepArena[i])
sqwPrepArena[i].Health:SetAllPoints()
sqwPrepArena[i].Health:SetStatusBarTexture(cfg.statusbar_texture)
sqwPrepArena[i].Health:SetStatusBarColor(.3, .3, .3, 1)
sqwPrepArena[i].SpecClass = sqwPrepArena[i].Health:CreateFontString(nil, "OVERLAY")
sqwPrepArena[i].SpecClass:SetFont(cfg.font, 9, "OUTLINE")
sqwPrepArena[i].SpecClass:SetPoint("CENTER")
sqwPrepArena[i]:Hide()
end

local ArenaListener = CreateFrame("Frame", "sqwArenaListener", UIParent)
ArenaListener:RegisterEvent("PLAYER_ENTERING_WORLD")
ArenaListener:RegisterEvent("ARENA_PREP_OPPONENT_SPECIALIZATIONS")
ArenaListener:RegisterEvent("ARENA_OPPONENT_UPDATE")
ArenaListener:SetScript("OnEvent", function(self, event)
if event == "ARENA_OPPONENT_UPDATE" then
for i=1, 5 do
local f = _G["sqwPrepArena"..i]
f:Hide()
end			
else
local numOpps = GetNumArenaOpponentSpecs()

if numOpps > 0 then
for i=1, 5 do
local f = _G["sqwPrepArena"..i]
local s = GetArenaOpponentSpec(i)
local _, spec, class = nil, "UNKNOWN", "UNKNOWN"

if s and s > 0 then 
_, spec, _, _, _, _, class = GetSpecializationInfoByID(s)
end

if (i <= numOpps) then
if class and spec then
f.SpecClass:SetText(spec.."  -  "..LOCALIZED_CLASS_NAMES_MALE[class])

local color = arena[i].colors.class[class]
f.Health:SetStatusBarColor(unpack(color))

f:Show()
end
else
f:Hide()
end
end
else
for i=1, 5 do
local f = _G["sqwPrepArena"..i]
f:Hide()
end			
end
end
end)

end

end)
