local addon, ns = ...
local cfg = ns.cfg
local cast = ns.cast
local lib = CreateFrame("Frame")
local _, playerClass = UnitClass("player")

ns.colors = setmetatable({
class = setmetatable({
["MONK"]		= {0.35, 0.98, 0.35 },
["HUNTER"]		= {0.29, 0.89, 0.05 },
["WARLOCK"]		= {0.49, 0.15, 0.8 },
["PRIEST"]		= {1.0, 1.0, 1.0 },
["PALADIN"]		= {0.96, 0.55, 0.73 },
["MAGE"]		= {0, 0.75, 1 },
["ROGUE"]		= {0.99, 0.82, 0.09 },
["DRUID"]		= {1.0, 0.47, 0.13 },
["SHAMAN"]		= {0.14, 0.35, 1.0 },
["WARRIOR"]		= {0.78, 0.61, 0.43 },
["DEATHKNIGHT"]	= {0.77, 0.12 , 0.23 },
}, {__index = oUF.colors.class}),

power = setmetatable({
['MANA'] 		= {0,0,1.0},--{0.18, 0.4, 1.0},
['RAGE'] 		= {1.0,0,0},
['FOCUS'] 		= {1.0,0.5,0.25},--{1.0,0.75,0.25},
['ENERGY']		= {1.0,1.0,0},--{1.0, 0.9, 0.35},
['RUNIC_POWER'] = {0.5,0.5,0.5},--{0.44,0.44,0.44},

}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})

-- FUNCTIONS
local retVal = function(f, val1, val2, val3)
if f.mystyle == "player" or f.mystyle == "target" then
return val1
elseif f.mystyle == "sqwRaid" or f.mystyle == "party" then
return val3
else
return val2
end
end

--status bar filling fix (from oUF_Mono)
local fixStatusbar = function(b)
b:GetStatusBarTexture():SetHorizTile(false)
b:GetStatusBarTexture():SetVertTile(false)
end

--backdrop table
local backdrop_tab = {
bgFile = cfg.backdrop_texture,
edgeFile = cfg.backdrop_edge_texture,
tile = false,
tileSize = 0,
edgeSize = 5,
insets = {
left = 3,
right = 3,
top = 3,
bottom = 3,
},
}

-- backdrop functions
lib.gen_backdrop = function(f)
f:SetBackdrop(backdrop_tab);
f:SetBackdropColor(0,0,0,0.2)
f:SetBackdropBorderColor(0,0,0,1)
end

lib.gen_powerbackdrop = function(f)
f:SetBackdrop(backdrop_tab);
f:SetBackdropColor(0,0,0,0.2)
f:SetBackdropBorderColor(0,0,0,1)
end

lib.gen_castbackdrop = function(f)
f:SetBackdrop(backdrop_tab);
f:SetBackdropColor(0,0,0,0.2)
f:SetBackdropBorderColor(0,0,0,1)
end

lib.gen_totemback = function(f)
f:SetBackdrop(backdrop_tab);
f:SetBackdropColor(0,0,0,0.2)
f:SetBackdropBorderColor(0,0,0,1)
end

lib.gen_expback = function(f)
f:SetBackdrop(backdrop_tab);
f:SetBackdropColor(0,0,0,0.2)
f:SetBackdropBorderColor(0,0,0,0.5)
end

lib.gen_classback = function(f)
f:SetBackdrop(backdrop_tab);
f:SetBackdropColor(0,0,0,0.1)
f:SetBackdropBorderColor(0,0,0,0.75)
end

-- Right Click Menu
lib.menu = function(self)
local unit = self.unit:sub(1, -2)
local cunit = self.unit:gsub("(.)", string.upper, 1)

if(cunit == 'Vehicle') then
cunit = 'Pet'
end

if(unit == "party" or unit == "partypet") then
ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
elseif(_G[cunit.."FrameDropDown"]) then
ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
end
end

--fontstring func
lib.gen_fontstring = function(f, name, size, outline)
local fs = f:CreateFontString(nil, "OVERLAY")
fs:SetFont(name, size, outline)
fs:SetShadowColor(0,0,0,0.8)
fs:SetShadowOffset(1,-1)
fs:SetWordWrap(disable)
return fs
end


function AlternativePowerOnToggle(self)
local unit = self:GetParent().unit or self:GetParent():GetParent().unit
end
function AlternativePowerPostUpdate(self, cur, min, max)
local perc = math.floor((cur/max)*100)
if perc < 35 then
self:SetStatusBarColor(0, 1, 0)
elseif perc < 70 then
self:SetStatusBarColor(1, 1, 0)
else
self:SetStatusBarColor(1, 0, 0)
end
local unit = self:GetParent().unit or self:GetParent():GetParent().unit
local type = select(10, UnitAlternatePowerInfo(unit))
end

--gen healthbar func
lib.gen_hpbar = function(f)
--statusbar
local s = CreateFrame("StatusBar", nil, f)
s:SetStatusBarTexture(cfg.statusbar_texture)
fixStatusbar(s)
--s:SetHeight(retVal(f,24,24,24))
s:SetHeight(retVal(f,13,11,25))
s:SetWidth(f:GetWidth())
s:SetPoint("TOP",0,0)
s:SetFrameLevel(3)
--helper
local h = CreateFrame("Frame", nil, s)
h:SetFrameLevel(2)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT", 5, -5)
lib.gen_backdrop(h)
--bg
local b = s:CreateTexture(nil, "BACKGROUND")
b:SetTexture(cfg.statusbar_texture)
b:SetAllPoints(s)
b:SetVertexColor(1, 0, 0,1)

f.Health = s
f.Health.bg2 = b
end

--gen hp strings func
lib.gen_hpstrings = function(f, unit)
--creating helper frame here so our font strings don't inherit healthbar parameters
local h = CreateFrame("Frame", nil, f)
h:SetAllPoints(f.Health)
h:SetFrameLevel(15)
local fontsize
if f.mystyle == "player" then fontsize = 20--18
elseif f.mystyle == "target" then fontsize = 20--18
elseif f.mystyle == "party" then fontsize = 12
elseif f.mystyle == "sqwRaid" then fontsize = 10
elseif f.mystyle == "partypet" then fontsize = 12
else fontsize = 20--14
end

local name = lib.gen_fontstring(f.Health, cfg.font, retVal(f,16,12,12), "THINOUTLINE")
--local name = lib.gen_fontstring(f.Health, cfg.font, retVal(f,16,16,16), "THINOUTLINE")
if f.mystyle == "player" then
name:SetPoint("LEFT", f.Health, "TOPLEFT", 0, 12)
name:SetJustifyH("LEFT")
elseif f.mystyle == "sqwRaid" then
name:SetPoint("LEFT", f.Health, "TOPLEFT", 0, -3)
name:SetJustifyH("LEFT")
elseif f.mystyle == "party" then
name:SetPoint("LEFT", f.Health, "TOPLEFT", 25, -6)
name:SetJustifyH("LEFT")
elseif f.mystyle == "pet" or f.mystyle == "partypet" then
name:SetPoint("LEFT", f.Health, "TOPLEFT", 0, 12)
name:SetJustifyH("LEFT")
else
name:SetPoint("LEFT", f.Health, "TOPLEFT", 0, 12)
name:SetJustifyH("LEFT")
end

local hpval = lib.gen_fontstring(f.Health, cfg.font, fontsize, "THINOUTLINE")

if f.mystyle == "party" --[[or f.mystyle == "pet" or f.mystyle == "partypet"]] then
hpval:SetPoint("CENTER", f.Health, "BOTTOM", 6, 5)
hpval:SetJustifyH("LEFT")
elseif f.mystyle == "sqwRaid" then
hpval:SetPoint("RIGHT", f.Health, "RIGHT", 6, -8)
hpval:SetJustifyH("LEFT")
elseif f.mystyle == "focus" or f.mystyle == "focustarget" then
hpval:SetPoint("CENTER", f.Health, "CENTER", 0, 0)
else
hpval:SetPoint("CENTER", f.Health, "CENTER", 0, -1)
end


if f.mystyle == "player" then
name:SetPoint("RIGHT", f, "RIGHT", 0, -1)
elseif f.mystyle == "sqwRaid" or f.mystyle == "party" then
name:SetPoint("CENTER", f, "CENTER", 0, 6)
elseif f.mystyle == "target" or f.mystyle == "pet" then
name:SetPoint("RIGHT", f, "RIGHT", 0, -1)
else
--name:SetPoint("RIGHT", hpval, "LEFT", -2, 0)
name:SetPoint("RIGHT", f, "RIGHT", 0, -1)
end


if f.mystyle == "player" then
f:Tag(name, "[sqw:level] [sqw:color][name]")
--f:Tag(name, "[sqw:pp]")
elseif f.mystyle == "target" then
f:Tag(name, "[sqw:level] [sqw:color][name]")
elseif f.mystyle == "sqwRaid" or f.mystyle == "party" then
f:Tag(name, "[sqw:color][name]")
else
f:Tag(name, "[sqw:color][name]")
end
if f.mystyle == "player" then
--f:Tag(hpval, "[sqw:color][sqw:hp]")
f:Tag(hpval, retVal(f,"[sqw:hp]","[sqw:hp]","[sqw:raidhp]"))
else
f:Tag(hpval, retVal(f,"[sqw:hp]","[sqw:hp]","[sqw:raidhp]"))
end


local level
if f.mystyle == "player" then level = lib.gen_fontstring(f.Health, cfg.font, 12, "THINOUTLINE")--lib.gen_fontstring(f.Health, cfg.font, fontsize/2, "THINOUTLINE")
elseif f.mystyle == "target" then level = lib.gen_fontstring(f.Health, cfg.font, 12, "THINOUTLINE")
elseif f.mystyle == "party" then level = lib.gen_fontstring(f.Health, cfg.font, 12, "THINOUTLINE")
end
if f.mystyle == "player" and cfg.ShowPlayerName then
--level:SetPoint("TOPRIGHT", f.Health, "TOPRIGHT", 0, 10)
level:SetJustifyH("RIGHT")
elseif f.mystyle =="target" and cfg.ShowTargetPower then
level:SetPoint("BOTTOMRIGHT", f.Health, "BOTTOMRIGHT", 22, -14)
level:SetJustifyH("RIGHT")
elseif f.mystyle=="party" then
--level:SetPoint("TOPRIGHT", f.Health, "TOPRIGHT", -1,-1)
level:SetPoint("TOPLEFT", f.Health, "TOPLEFT", 1,0)
level:SetJustifyH("RIGHT")
end
if f.mystyle == "player" and cfg.ShowPlayerName then
f:Tag(level, "[sqw:level] [sqw:color][name]")
elseif f.mystyle =="target" and cfg.ShowTargetPower then
f:Tag(level, "[sqw:pp]")
elseif f.mystyle =="party" then
f:Tag(level, "[sqw:level]")
end
end


--gen powerbar func
lib.gen_ppbar = function(f)
--statusbar
local s = CreateFrame("StatusBar", nil, f)
s:SetStatusBarTexture(cfg.powerbar_texture)
s:GetStatusBarTexture():SetHorizTile(true)
s:SetWidth(f:GetWidth())
--s:SetHeight(retVal(f,12,10,10))
s:SetHeight(6)


if f.mystyle == "party" --[["target" or f.mystyle == "tot" or f.mystyle == "focustarget"]] then
s:SetPoint("BOTTOM",f,"BOTTOM",0,-1)
else
s:SetPoint("BOTTOM",f,"BOTTOM",0,5)
end	s:SetFrameLevel(1)
s:SetFrameLevel(2)


--helper
local h = CreateFrame("Frame", nil, s)
h:SetFrameLevel(0)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_powerbackdrop(h)
--bg
local b = s:CreateTexture(nil, "BACKGROUND")
b:SetTexture(cfg.powerbar_texture)
b:SetVertexColor(0, 0, 0,0.5)
b:SetAllPoints(s)
f.Power = s
f.Power.bg = b
f.Power = s
f.Power.bg = b
end

--gen combat and LFD icons
lib.gen_InfoIcons = function(f)
local h = CreateFrame("Frame",nil,f)
h:SetAllPoints(f)
h:SetFrameLevel(10)
--combat icon
if f.mystyle == 'player' then
f.CombatIndicator = h:CreateTexture(nil, 'OVERLAY')
f.CombatIndicator:SetSize(15,15)
f.CombatIndicator:SetPoint('RIGHT', 20, 10)
f.CombatIndicator:SetTexture([[Interface\Addons\oUF_sqw\media\combat]])
end
-- rest icon
if f.mystyle == 'player' and UnitLevel("Player") < 100 then
f.RestingIndicator = h:CreateTexture(nil, 'OVERLAY')
f.RestingIndicator:SetSize(22,22)
f.RestingIndicator:SetPoint('BOTTOMLEFT', -3, -3)
f.RestingIndicator:SetTexture([[Interface\Addons\oUF_sqw\media\resting]])
f.RestingIndicator:SetAlpha(0.75)
end
--LeaderIndicator icon
li = h:CreateTexture(nil, "OVERLAY")
li:SetPoint("TOPLEFT", f, 10, 10)
li:SetSize(12,12)
f.LeaderIndicator = li


--Assist icon
ai = h:CreateTexture(nil, "OVERLAY")
ai:SetPoint("TOPLEFT", f, 10, 8)
ai:SetSize(12,12)
f.AssistantIndicator = ai
--ML icon
local ml = h:CreateTexture(nil, 'OVERLAY')
ml:SetSize(12,12)
ml:SetPoint('LEFT', f.LeaderIndicator, 'RIGHT')
f.MasterLooterIndicator = ml
end

-- LFG Role Indicator
lib.gen_GroupRoleIndicator = function(f)
local lfdi = lib.gen_fontstring(f.Health, cfg.smallfont, 10, "THINOUTLINE")
lfdi:SetPoint('BOTTOM', f.Health, 'TOP', 0, 4)
f:Tag(lfdi, "[sqw:lfdrole]")
end

-- phase icon
lib.addPhaseIndicator = function(self)
local picon = self.Health:CreateTexture(nil, 'OVERLAY')
picon:SetPoint('TOPRIGHT', self, 'TOPRIGHT', 40, 8)
picon:SetSize(16, 16)

self.PhaseIndicator = picon
end

-- quest icon
lib.addQuestIndicator = function(self)
local qicon = self.Health:CreateTexture(nil, 'OVERLAY')
qicon:SetPoint('TOPLEFT', self, 'TOPLEFT', 0, 8)
qicon:SetSize(16, 16)

self.QuestIndicator = qicon
end

--gen raid mark icons
lib.gen_RaidMark = function(f)
local h = CreateFrame("Frame", nil, f)
h:SetAllPoints(f)
h:SetFrameLevel(10)
h:SetAlpha(0.8)
local ri = h:CreateTexture(nil,'OVERLAY',h)
ri:SetPoint("TOPRIGHT", f, "TOPRIGHT", -20, 5)
ri:SetTexture([[Interface\Addons\oUF_sqw\media\raidicons.blp]])
local size = retVal(f, 19, 19, 19)
ri:SetSize(size, size)
f.RaidTargetIndicator = ri
end

--gen highlight texture
lib.gen_highlight = function(f)
local OnEnter = function(f)
UnitFrame_OnEnter(f)
f.Highlight:Show()
end
local OnLeave = function(f)
UnitFrame_OnLeave(f)
f.Highlight:Hide()
end
f:SetScript("OnEnter", OnEnter)
f:SetScript("OnLeave", OnLeave)
local hl = f.Health:CreateTexture(nil, "OVERLAY")
hl:SetAllPoints(f.Health)
hl:SetTexture(cfg.statusbar_texture)
hl:SetVertexColor(.5,.5,.5,.1)
hl:SetBlendMode("ADD")
hl:Hide()
f.Highlight = hl
end

-- Create Target Border
function lib.CreateTargetBorder(self)
self.TargetBorder = self.Health:CreateTexture("BACKGROUND", nil, self)
self.TargetBorder:SetPoint("TOPLEFT", self.Health, "TOPLEFT", -24, 24)
self.TargetBorder:SetPoint("BOTTOMRIGHT", self.Health, "BOTTOMRIGHT", 24, -24)
self.TargetBorder:SetTexture([[Interface\Addons\oUF_sqw\media\target]])
self.TargetBorder:SetVertexColor(1.0, 1.0, 0.1,0.6)
self.TargetBorder:SetBlendMode("ADD")
self.TargetBorder:Hide()
end

-- Raid Frames Target Highlight Border
function lib.ChangedTarget(self, event, unit)

if (UnitIsUnit('target', 'player')) then
self.TargetBorder:Show()
else
self.TargetBorder:Hide()
end
end

-- Create Raid ThreatIndicator Status Border
function lib.CreateThreatIndicatorBorder(self)

local glowBorder = {edgeFile = cfg.backdrop_edge_texture, edgeSize = 5}
self.Thtborder = CreateFrame("Frame", nil, self)
self.Thtborder:SetPoint("TOPLEFT", self, "TOPLEFT", -8, 9)
self.Thtborder:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 9, -9)
self.Thtborder:SetBackdrop(glowBorder)
self.Thtborder:SetFrameLevel(1)
self.Thtborder:Hide()
end

-- Raid Frames ThreatIndicator Highlight
function lib.UpdateThreat(self, event, unit)

if (self.unit ~= unit) then return end

local status = UnitThreatSituation(unit)
unit = unit or self.unit

if status and status > 1 then
local r, g, b = GetThreatStatusColor(status)
self.Thtborder:Show()
self.Thtborder:SetBackdropBorderColor(r, g, b, 1)
else
self.Thtborder:SetBackdropBorderColor(r, g, b, 0)
self.Thtborder:Hide()
end
end

-- Castbar
local PostCastStart = function(castbar, unit)
if unit ~= 'player' then
if castbar.interrupt then
castbar.Backdrop:SetBackdropBorderColor(1, .9, .4)
castbar.Backdrop:SetBackdropColor(1, .9, .4)
else
castbar.Backdrop:SetBackdropBorderColor(0, 0, 0, 0)
castbar.Backdrop:SetBackdropColor(0, 0, 0)
end
end
end

local CustomTimeText = function(castbar, duration)
if castbar.casting then
castbar.Time:SetFormattedText("%.1f / %.1f", duration, castbar.max)
elseif castbar.channeling then
castbar.Time:SetFormattedText("%.1f / %.1f", castbar.max - duration, castbar.max)
end
end

lib.gen_castbar = function(f)
if not cfg.Castbars then return end
local cbColor = {95/255, 95/255, 255/255, 1}
local s = CreateFrame("StatusBar", "oUF_sqwCastbar"..f.mystyle, f)
s:SetHeight(12)
s:SetAlpha(1)
if f.mystyle == "focus" then
s:SetWidth(f:GetWidth()-2)
elseif f.mystyle == "player" then
s:SetWidth(f:GetWidth())
else
s:SetWidth(f:GetWidth())
end

if f.mystyle == "player" then
s:SetPoint("BOTTOMRIGHT",f,"BOTTOMRIGHT",0,-15)
elseif f.mystyle == "target" then
s:SetPoint("BOTTOMRIGHT",f,"BOTTOMRIGHT",0,-15)
elseif f.mystyle == "targettarget" then
s:SetPoint("BOTTOM",f,"TOP",0,0)
elseif f.mystyle == "sqwBoss" then
s:SetPoint("BOTTOMRIGHT",f,"BOTTOMRIGHT",0,-5)
else
s:SetPoint("TOPLEFT",f,"BOTTOMLEFT",0,-15)
end
s:SetStatusBarTexture(cfg.statusbar_texture)
s:SetStatusBarColor{95/255, 95/255, 255/255,0.1}
s:SetFrameLevel(2)
--color
s.CastingColor = cbColor
s.CompleteColor = {95/255, 95/255, 255/255,0.1}
s.FailColor = {255/255, 12/255, 0/255,0.1}
s.ChannelingColor = cbColor
--helper
local h = CreateFrame("Frame", nil, s)
h:SetFrameLevel(0)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_castbackdrop(h)
--spark
sp = s:CreateTexture(nil, "OVERLAY")
sp:SetTexture(spark)
sp:SetBlendMode("ADD")
sp:SetVertexColor(1, 1, 1, 1)
sp:SetHeight(s:GetHeight()*2.5)
sp:SetWidth(s:GetWidth()/18)
--spell text
local txt = lib.gen_fontstring(s, cfg.font, 14, "NONE")
txt:SetPoint("LEFT", 2, 0)
-- txt:SetPoint("LEFT", 2, 10)
txt:SetJustifyH("LEFT")
--time
local t = lib.gen_fontstring(s, cfg.font, 10, 'NONE')
t:SetPoint("RIGHT", -2, 0)
txt:SetPoint("RIGHT", f, "RIGHT", 0, 0)
--icon
local i = s:CreateTexture(nil, "ARTWORK")
i:SetSize(25,25)
i:SetPoint("BOTTOMLEFT", s, "BOTTOMRIGHT", 9, 0)
i:SetTexCoord(0.1, 0.9, 0.1, 0.9)
--helper2 for icon
local h2 = CreateFrame("Frame", nil, s)
h2:SetFrameLevel(0)
h2:SetPoint("TOPLEFT",i,"TOPLEFT",-5,5)
h2:SetPoint("BOTTOMRIGHT",i,"BOTTOMRIGHT",5,-5)
lib.gen_backdrop(h2)
if f.mystyle == "player" then
--latency (only for player unit)
local z = s:CreateTexture(nil,"OVERLAY")
z:SetTexture(cfg.statusbar_texture)
z:SetVertexColor(1,0.1,0,.6)
z:SetPoint("TOPRIGHT")
z:SetPoint("BOTTOMRIGHT")
s:SetFrameLevel(1)
s.SafeZone = z
-- custom latency display
local l = lib.gen_fontstring(s, cfg.font, 10, "THINOUTLINE")
l:SetPoint("CENTER", -2, 17)
l:SetJustifyH("RIGHT")
l:Hide()
s.Lag = l
f:RegisterEvent("UNIT_SPELLCAST_SENT", cast.OnCastSent)
end
s.OnUpdate = cast.OnCastbarUpdate
s.PostCastStart = cast.PostCastStart
s.PostChannelStart = cast.PostCastStart
s.PostCastStop = cast.PostCastStop
s.PostChannelStop = cast.PostChannelStop
s.PostCastFailed = cast.PostCastFailed
s.PostCastInterrupted = cast.PostCastFailed

f.Castbar = s
f.Castbar.Text = txt
f.Castbar.Time = t
f.Castbar.Icon = i
f.Castbar.Spark = sp
end


-- mirror castbar!
lib.gen_mirrorcb = function(f)
for _, bar in pairs({'MirrorTimer1','MirrorTimer2','MirrorTimer3',}) do
for i, region in pairs({_G[bar]:GetRegions()}) do
if (region.GetTexture and region:GetTexture() == 'SolidTexture') then
region:Hide()
end
end
_G[bar..'Border']:Hide()
_G[bar]:SetParent(UIParent)
_G[bar]:SetScale(1)
_G[bar]:SetHeight(16)
_G[bar]:SetWidth(280)
_G[bar]:SetBackdropColor(.1,.1,.1)
_G[bar..'Background'] = _G[bar]:CreateTexture(bar..'Background', 'BACKGROUND', _G[bar])
_G[bar..'Background']:SetTexture(cfg.statusbar_texture)
_G[bar..'Background']:SetAllPoints(bar)
_G[bar..'Background']:SetVertexColor(.15,.15,.15,.75)
_G[bar..'Text']:SetFont(cfg.font, 14)
_G[bar..'Text']:ClearAllPoints()
_G[bar..'Text']:SetPoint('CENTER', MirrorTimer1StatusBar, 0, 1)
_G[bar..'StatusBar']:SetAllPoints(_G[bar])
--glowing borders
local h = CreateFrame("Frame", nil, _G[bar])
h:SetFrameLevel(0)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_backdrop(h)
end
end

-- Post Create Icon Function
local myPostCreateIcon = function(self, button)

self.showDebuffType = true
self.showBuffType = true
--self.showStealableBuffs = true
self.disableCooldown = true
button.cd.noOCC = true
button.cd.noCooldownCount = true

--		button.icon:SetTexCoord(0,1,0,1)
button.icon:SetTexCoord(.07, .93, .07, .93)
button.icon:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
button.icon:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
button.overlay:SetTexture(cfg.debuffBorder)
button.overlay:SetTexCoord(0,1,0,1)
button.overlay.Hide = function(self) self:SetVertexColor(0.3, 0.3, 0.3) end


button.time = lib.gen_fontstring(button, cfg.smallfont, 12, "OUTLINE")
button.time:SetPoint("BOTTOM", button, 2, -6)
button.time:SetJustifyH('CENTER')
button.time:SetVertexColor(1,1,1)

button.count = lib.gen_fontstring(button, cfg.smallfont, 15, "OUTLINE")
button.count:ClearAllPoints()
button.count:SetPoint("TOPRIGHT", button, 5, 3)
button.count:SetJustifyH('RIGHT')
button.count:SetVertexColor(1,1,1)

-- helper
local h = CreateFrame("Frame", nil, button)
h:SetFrameLevel(0)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_expback(h)
end

-- Post Update Icon Function
local myPostUpdateIcon = function(self, unit, icon, index, offset, filter, isDebuff)

local _, _, _, _, _, duration, expirationTime, unitCaster, _ = UnitAura(unit, index, icon.filter)

if duration and duration > 0 then
icon.time:Show()
icon.timeLeft = expirationTime
icon:SetScript("OnUpdate", CreateBuffTimer)
else
icon.time:Hide()
icon.timeLeft = math.huge
icon:SetScript("OnUpdate", nil)
end

-- Desaturate non-Player Debuffs
if(icon.isDebuff) then
if(unit == "target") then
if (unitCaster == 'player' or unitCaster == 'vehicle') then
icon.icon:SetDesaturated(false)
elseif(not UnitPlayerControlled(unit)) then -- If Unit is Player Controlled don't desaturate debuffs
icon:SetBackdropColor(0, 0, 0)
icon.overlay:SetVertexColor(0.3, 0.3, 0.3)
icon.icon:SetDesaturated(true)
end
end
end

-- Right Click Cancel Buff/Debuff
icon:SetScript('OnMouseUp', function(self, mouseButton)
if mouseButton == 'RightButton' then
CancelUnitBuff('player', index)
end end)

icon.first = true
end

local FormatTime = function(s)
local day, hour, minute = 86400, 3600, 60
if s >= day then
return format("%dd", floor(s/day + 0.5)), s % day
elseif s >= hour then
return format("%dh", floor(s/hour + 0.5)), s % hour
elseif s >= minute then
if s <= minute * 5 then
return format("%d:%02d", floor(s/60), s % minute), s - floor(s)
end
return format("%dm", floor(s/minute + 0.5)), s % minute
elseif s >= minute / 12 then
return floor(s + 0.5), (s * 100 - floor(s * 100))/100
end
return format("%.1f", s), (s * 100 - floor(s * 100))/100
end

-- Create Buff/Debuff Timer Function
function CreateBuffTimer(self, elapsed)
self.elapsed = (self.elapsed or 0) + elapsed
if self.elapsed >= 0.1 then
if not self.first then
self.timeLeft = self.timeLeft - self.elapsed
else
self.timeLeft = self.timeLeft - GetTime()
self.first = false
end
if self.timeLeft > 0 and self.timeLeft <= 60*15 then -- Show time between 0 and 15 min
local time = FormatTime(self.timeLeft)
self.time:SetText(time)
if self.timeLeft >= 6 and self.timeLeft <= 60*5 then -- if Between 5 min and 6sec
self.time:SetTextColor(0.95,0.95,0.95)
elseif self.timeLeft > 3 and self.timeLeft < 6 then -- if Between 6sec and 3sec
self.time:SetTextColor(0.95,0.70,0)
elseif self.timeLeft <= 3 then -- Below 3sec
self.time:SetTextColor(0.9, 0.05, 0.05)
else
self.time:SetTextColor(0.95,0.95,0.95) -- Fallback Color
end
else
self.time:Hide()
end
self.elapsed = 0
end
end

-- Generates the Buffs
lib.createBuffs = function(f)
b = CreateFrame("Frame", nil, f)
b.onlyShowPlayer = cfg.buffsOnlyShowPlayer
if f.mystyle == "target" then
b.size = 25
b.num = 14
b.spacing = 5
if(playerClass == "ROGUE" or playerClass == "DRUID") then
b:SetPoint("BOTTOMLEFT", f, "TOPLEFT", 2, 30)
else
b:SetPoint("BOTTOMLEFT", f, "TOPLEFT", 6, -50)
end
b.initialAnchor = "BOTTOMLEFT"
b["growth-x"] = "RIGHT"
b["growth-y"] = "UP"
b:SetHeight((b.size+b.spacing)*4)
b:SetWidth(f:GetWidth())
else
b.num = 0
end
b.PostCreateIcon = myPostCreateIcon
b.PostUpdateIcon = myPostUpdateIcon
f.Buffs = b
end

-- Generates the Debuffs
lib.createDebuffs = function(f)
b = CreateFrame("Frame", nil, f)
if f.mystyle == "tot" or f.mystyle == "focus" then
b.onlyShowPlayer = false
b.size = 25
b.num = 8
else
b.onlyShowPlayer = cfg.showTargetDebuffs
b.size = 25
b.num = 12
end
b.spacing = 6
b:SetHeight((b.size+b.spacing)*5)
b:SetWidth(f:GetWidth())
b:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 55)
--b:SetPoint("TOPLEFT", f, "TOPRIGHT", 25, 0)
b.initialAnchor = "TOPLEFT"
b["growth-x"] = "RIGHT"
b["growth-y"] = "up"
b.PostCreateIcon = myPostCreateIcon
b.PostUpdateIcon = myPostUpdateIcon
f.Debuffs = b
end

-- raid post update
lib.PostUpdateRaidFrame = function(Health, unit, min, max)

local disconnnected = not UnitIsConnected(unit)
local dead = UnitIsDead(unit)
local ghost = UnitIsGhost(unit)
if disconnnected or dead or ghost then
Health:SetValue(max)
if(disconnnected) then
Health:SetStatusBarColor(0,0,0,0.6)
elseif(ghost) then
Health:SetStatusBarColor(1,1,1,0.6)
elseif(dead) then
Health:SetStatusBarColor(1,0,0,0.7)
end
else
Health:SetValue(min)
if(unit == 'vehicle') then
Health:SetStatusBarColor(22/255, 106/255, 44/255)
end
end
if not UnitInRange(unit) then
Health.bg2:SetVertexColor(.6, 0.3, 0.3,1)
else
Health.bg2:SetVertexColor(1, 0.1, 0.1,1)
end
end

lib.genRunes = function(self)
if playerClass ~= "DEATHKNIGHT" then return end

local runeFrame = CreateFrame("Frame", nil, self)
runeFrame:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -5)
runeFrame:SetHeight(4)
runeFrame:SetWidth(self:GetWidth())
runeFrame:SetFrameLevel(4)

for i= 1, 6 do
local rune = CreateFrame("StatusBar", nil, runeFrame)
rune:SetSize((self.Health:GetWidth() / 6)-6, 6)
rune:SetStatusBarTexture(cfg.statusbar_texture)
rune:SetFrameLevel(4)

local h = CreateFrame("Frame", nil, rune)
h:SetFrameLevel(1)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_totemback(h)

if (i == 1) then
rune:SetPoint("BOTTOMLEFT", self.Health, "TOPLEFT", 3, 6)
else
rune:SetPoint("TOPLEFT", runeFrame[i-1], "TOPRIGHT", 6, 0)
end

runeFrame[i] = rune
end
self.Runes = runeFrame
end

-- Class specific powers
lib.gen_AlternativePower = function(self)

local AdditionalPower = CreateFrame("StatusBar", "AdditionalPowerBar", self.Power)
AdditionalPower:SetHeight(6)
AdditionalPower:SetWidth(self.Power:GetWidth()-20)
AdditionalPower:SetPoint("TOP", self.Power, "BOTTOM", 0, 0)
AdditionalPower:SetFrameLevel(2)
AdditionalPower:SetStatusBarTexture(cfg.statusbar_texture)
AdditionalPower:SetStatusBarColor(.117, .55, 1)

AdditionalPower.bg = AdditionalPower:CreateTexture(nil, "BORDER")
AdditionalPower.bg:SetTexture(cfg.statusbar_texture)
AdditionalPower.bg:SetVertexColor(1, 0.1, 0.1,1)
AdditionalPower.bg:SetPoint("TOPLEFT", AdditionalPower, "TOPLEFT", 0, 0)
AdditionalPower.bg:SetPoint("BOTTOMRIGHT", AdditionalPower, "BOTTOMRIGHT", 0, 0)

local h = CreateFrame("Frame", nil, AdditionalPower)
h:SetFrameLevel(1)
h:SetPoint("TOPLEFT", -5, 5)
h:SetPoint("BOTTOMRIGHT", 5, -5)
lib.gen_backdrop(h)

self.AdditionalPower = AdditionalPower
self.AdditionalPower.bg = AdditionalPower.bg
end

-- Runebar
lib.genRunes = function(self)
if playerClass ~= "DEATHKNIGHT" then return end
local Runes = CreateFrame("Frame", nil, self)
Runes:SetPoint('CENTER', self.Health, 'TOP', 2, 1)
Runes:SetHeight(3)
Runes:SetWidth(self.Health:GetWidth())

for i= 1, 6 do
Runes[i] = CreateFrame("StatusBar", self:GetName().."_Runes"..i, self)
Runes[i]:SetHeight(8)
Runes[i]:SetWidth((self.Health:GetWidth() / 6)-5)
Runes[i]:SetStatusBarTexture(cfg.statusbar_texture)
Runes[i]:SetFrameLevel(10)
Runes[i]:SetStatusBarColor(70/255, 180/255, 210/255)
Runes[i].bg = Runes[i]:CreateTexture(nil, "BORDER")
Runes[i].bg:SetTexture(cfg.statusbar_texture)
Runes[i].bg:SetPoint("TOPLEFT", Runes[i], "TOPLEFT", 0, 0)
Runes[i].bg:SetPoint("BOTTOMRIGHT", Runes[i], "BOTTOMRIGHT", 0, 0)
Runes[i].bg.multiplier = 0.2

local h = CreateFrame("Frame", nil, Runes[i])
h:SetFrameLevel(1)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_classback(h)

if (i == 1) then
Runes[i]:SetPoint("BOTTOMLEFT", self.Health, "TOPLEFT", 0, 8)
else
Runes[i]:SetPoint('TOPLEFT', Runes[i-1], 'TOPRIGHT', 1, 0)
end
end

self.Runes = Runes
end
-- Class Power
lib.gen_Classbar = function(self)
local maxPower, color
if playerClass == "MAGE" then
maxPower = 4
color = {0.15, 0.55, 0.8}
elseif playerClass == "MONK" then
maxPower = 6
color = {0.9, 0.99, 0.9}
elseif playerClass == "PALADIN" then
maxPower = 5
color = {0.9, 0.95, 0.33}
elseif playerClass == "WARLOCK" then
maxPower = 5
color = {0.86, 0.22, 1}
end

if maxPower ~= nil then
local ClassPower = CreateFrame("Frame", nil, self)
ClassPower:SetPoint('BOTTOM', self.Health, 'TOP', 130, 5)
ClassPower:SetHeight(10)
ClassPower:SetWidth(self.Health:GetWidth()/3)

ClassPower:SetFrameLevel(10)

for i = 1, maxPower do
ClassPower[i] = CreateFrame("StatusBar", self:GetName()..playerClass..i, self)
ClassPower[i]:SetHeight(10)
ClassPower[i]:SetWidth(10)
--[[ClassPower[i]:SetHeight(4)
ClassPower[i]:SetWidth((ClassPower:GetWidth() / maxPower) - 2.5)]]
ClassPower[i]:SetStatusBarTexture(cfg.statusbar_texture)
ClassPower[i]:SetStatusBarColor(color[1], color[2], color[3])
ClassPower[i]:SetFrameLevel(11)

local h = CreateFrame("Frame", nil, ClassPower[i])
h:SetFrameLevel(10)
h:SetPoint("TOPLEFT", -5, 5)
h:SetPoint("BOTTOMRIGHT", 5, -5)
lib.gen_classback(h)

if (i == 1) then
--ClassPower[i]:SetPoint("BOTTOMLEFT", self.Health, "TOPLEFT", 0, 8)
ClassPower[i]:SetPoint("TOPRIGHT", self.Health, "TOPRIGHT", -75, 17)

else
ClassPower[i]:SetPoint('TOPLEFT', ClassPower[i-1], "TOPRIGHT", 8, 0)
end
end

self.ClassPower = ClassPower
end
end

-- Combo points
lib.RogueComboPoints = function(self)
if(playerClass == "ROGUE" or playerClass == "DRUID") then

local combo = CreateFrame("Frame", nil, self)
combo:SetPoint('BOTTOM', self.Health, 'TOP', 160, 1)--SetPoint("TOPLEFT", self.Health, "BOTTOMLEFT", 0, -5)
combo:SetHeight(3)
combo:SetWidth(self.Health:GetWidth()/3)--(self:GetWidth())

for i= 1, 8 do
combo[i] = CreateFrame("StatusBar", self:GetName().."_ClassPower"..i, self)
combo[i]:SetHeight(6)
combo[i]:SetStatusBarTexture(cfg.statusbar_texture)
combo[i]:SetFrameLevel(10)
combo[i].bg = combo[i]:CreateTexture(nil, "BORDER")
combo[i].bg:SetTexture(cfg.statusbar_texture)
combo[i].bg:SetPoint("TOPLEFT", combo[i], "TOPLEFT", 0, 0)
combo[i].bg:SetPoint("BOTTOMRIGHT", combo[i], "BOTTOMRIGHT", 0, 0)
combo[i].bg.multiplier = 0.3

local h = CreateFrame("Frame", nil, combo[i])
h:SetFrameLevel(1)
h:SetPoint("TOPLEFT", -5, 5)
h:SetPoint("BOTTOMRIGHT", 5, -5)
lib.gen_classback(h)

if (i == 1) then
combo[i]:SetPoint("BOTTOMLEFT", self.Health, "TOPLEFT", 150, 8)   --druid xy points
else
combo[i]:SetPoint('TOPLEFT', combo[i-1], 'TOPRIGHT', 3, 0)
end
end

combo[1]:SetStatusBarColor(.3,.9,.3)
combo[2]:SetStatusBarColor(.3,.9,.3)
combo[3]:SetStatusBarColor(.3,.9,.3)
combo[4]:SetStatusBarColor(.9,.9,0)
combo[5]:SetStatusBarColor(.9,.3,.3)
combo[6]:SetStatusBarColor(.9,.3,.3)
combo[7]:SetStatusBarColor(.9,.3,.3)
combo[8]:SetStatusBarColor(.9,.3,.3)

self.FailClassPower = combo
end
end


-- ReadyCheckIndicator
lib.ReadyCheckIndicator = function(self)
if cfg.RCheckIcon then
rCheck = self.Health:CreateTexture(nil, "OVERLAY")
rCheck:SetSize(14, 14)
rCheck:SetPoint("BOTTOMLEFT", self.Health, "TOPRIGHT", -13, -12)
self.ReadyCheckIndicator = rCheck
end
end

-- raid debuffs
lib.raidDebuffs = function(f)
if cfg.showRaidDebuffs then
local raid_debuffs = {
debuffs = {
-- Any Zone
["Viper Sting"] = 12, 		-- Viper Sting
["Wound Poison"] = 9,		-- Wound Poison
["Mortal Strike"] = 8,		-- Mortal Strike
["Furious Attacks"] = 8, 	-- Furious Attacks
["Aimed Shot"] = 8, 		-- Aimed Shot
["Counterspell"] = 10,		-- Counterspell
["Blind"] = 10,				-- Blind
["Cyclone"] = 10, 			-- Cyclone
["Hex"] = 7,				-- Hex
["Polymorph"] = 7, 			-- Polymorph
["Entangling Roots"] = 7, 	-- Entangling Roots
["Frost Nova"] = 7,			-- Frost Nova
["Freezing Trap"] = 7, 		-- Freezing Trap
["Crippling Poison"] = 6, 	-- Crippling Poison
["Bash"] = 5,				-- Bash
["Cheap Shot"] = 5,			-- Cheap Shot
["Kidney Shot"] = 5,		-- Kidney Shot
["Throwdown"] = 5,			-- Throwdown
["Sap"] = 5,				-- Sap
["Hamstring"] = 5, 			-- Hamstring
["Wing Clip"] = 5, 			-- Wing Clip
["Fear"] = 3, 				-- Fear
["Psychic Scream"] = 3, 	-- Psychic Scream
["Howl of Terror"] = 3, 	-- Howl of Terror
["Intimidating Shout"] = 3, -- Intimidating Shout

},
}

local instDebuffs = {}
local instances = raid_debuffs.instances
local getzone = function()
local zone = GetInstanceInfo()
if instances[zone] then
instDebuffs = instances[zone]
else
instDebuffs = {}
end
end

local debuffs = raid_debuffs.debuffs
local CustomFilter = function(icons, ...)
local _, icon, name, _, _, _, dtype = ...
if instDebuffs[name] then
icon.priority = instDebuffs[name]
return true
elseif debuffs[name] then
icon.priority = debuffs[name]
return true
else
icon.priority = 0
end
end

local dbsize = 18
local debuffs = CreateFrame("Frame", nil, f)
debuffs:SetWidth(dbsize) debuffs:SetHeight(dbsize)
debuffs:SetPoint("topLEFT", 0, 20) --DEBUFF START ANCHOR POINT
debuffs.size = dbsize

debuffs.CustomFilter = CustomFilter
f.raidDebuffs = debuffs
end
end

-- oUF_HealPred
lib.HealPred = function(self)
if not cfg.ShowIncHeals then return end

local mhpb = CreateFrame('StatusBar', nil, self.Health)
mhpb:SetPoint('TOPLEFT', self.Health:GetStatusBarTexture(), 'TOPRIGHT', 0, 0)
mhpb:SetPoint('BOTTOMLEFT', self.Health:GetStatusBarTexture(), 'BOTTOMRIGHT', 0, 0)
mhpb:SetWidth(self:GetWidth())
mhpb:SetStatusBarTexture(cfg.statusbar_texture)
mhpb:SetStatusBarColor(1, 1, 1, 0.4)
mhpb:SetFrameLevel(1)

local ohpb = CreateFrame('StatusBar', nil, self.Health)
ohpb:SetPoint('TOPLEFT', mhpb:GetStatusBarTexture(), 'TOPRIGHT', 0, 0)
ohpb:SetPoint('BOTTOMLEFT', mhpb:GetStatusBarTexture(), 'BOTTOMRIGHT', 0, 0)
ohpb:SetWidth(self:GetWidth())
ohpb:SetStatusBarTexture(cfg.statusbar_texture)
ohpb:SetStatusBarColor(1, 1, 1, 0.4)
mhpb:SetFrameLevel(1)
self.HealthPrediction = {
myBar = mhpb,
otherBar = ohpb,
maxOverflow = 1,
}
end

-- Combo points
lib.RogueComboPoints = function(self)
if(playerClass == "ROGUE" or playerClass == "DRUID") then

local barFrame = CreateFrame("Frame", nil, self)
barFrame:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -5)
barFrame:SetHeight(4)
barFrame:SetWidth(self:GetWidth())
barFrame:SetFrameLevel(4)

for i = 1, MAX_COMBO_POINTS do
local point = CreateFrame("StatusBar", nil, barFrame)
point:SetSize((self.Health:GetWidth() / 5)-4.5, 6)
if i > 1 then point:SetPoint("LEFT", point[i - 1], "RIGHT") end
point:SetStatusBarTexture(cfg.statusbar_texture)
point:SetStatusBarColor(1.0, 0.9, 0)

local h = CreateFrame("Frame", nil, point)
h:SetFrameLevel(1)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_totemback(h)

if (i == 1) then
point:SetPoint('BOTTOMLEFT', self.Health, 'TOPLEFT', 5, 13)
else
point:SetPoint("TOPLEFT", barFrame[i-1], "TOPRIGHT", 6, 0)
end
barFrame[i] = point
end
self.ClassPower = barFrame
self.ClassPower.unit = "player"
end
end

-- Addons/Plugins -------------------------------------------

-- Totem timer
lib.gen_TotemBar = function(self)
if playerClass ~= "SHAMAN" then return end
local totems = CreateFrame("Frame", nil, self)
totems:SetPoint("TOPLEFT", self, "TOPLEFT",4,-3)
totems:SetWidth(self:GetWidth()-140)
totems:SetHeight(6)
totems:SetFrameLevel(6)
totems.Destroy = true
totems.colors = {{233/255, 46/255, 16/255};{173/255, 217/255, 25/255};{35/255, 127/255, 255/255};{178/255, 53/255, 240/255};}

for i = 1, 4 do
totems[i] = CreateFrame("StatusBar", nil, totems)
totems[i]:SetHeight(totems:GetHeight())
totems[i]:SetWidth(((self:GetWidth()-140) - 3) / 4)

if (i == 1) then
totems[i]:SetPoint("LEFT", totems)
else
totems[i]:SetPoint("LEFT", totems[i-1], "RIGHT", 1, 0)
end
totems[i]:SetStatusBarTexture(cfg.statusbar_texture)
totems[i]:GetStatusBarTexture():SetHorizTile(false)
totems[i]:SetMinMaxValues(0, 1)

totems[i].bg = totems[i]:CreateTexture(nil, "BORDER")
totems[i].bg:SetAllPoints()
totems[i].bg:SetTexture(cfg.statusbar_texture)
totems[i].bg.multiplier = 0.3
end
totems.backdrop = CreateFrame("Frame", nil, totems)

--[[CreateShadowclassbar(totems.backdrop)
totems.backdrop:SetBackdropBorderColor(.2,.2,.2,1)
totems.backdrop:SetPoint("TOPLEFT", -2, 2)
totems.backdrop:SetPoint("BOTTOMRIGHT", 2, -2)
totems.backdrop:SetFrameLevel(5) ]]
self.TotemBar = totems
end



-- oUF_DebuffHighlight
lib.debuffHighlight = function(self)
if cfg.enableDebuffHighlight then
local dbh = self.Health:CreateTexture(nil, "OVERLAY")
dbh:SetAllPoints(self.Health)
dbh:SetTexture(cfg.debuffhighlight_texture)
dbh:SetBlendMode("ADD")
dbh:SetVertexColor(0,0,0,0) -- set alpha to 0 to hide the texture
self.DebuffHighlight = dbh
self.DebuffHighlightAlpha = 0.5
self.DebuffHighlightFilter = true
end
end

-- AuraWatch
local AWPostCreateIcon = function(AWatch, icon, spellID, name, self)
icon.cd:SetReverse()
local count = lib.gen_fontstring(icon, cfg.smallfont, 12, "OUTLINE")
count:SetPoint("CENTER", icon, "BOTTOM", 3, 3)
icon.count = count
local h = CreateFrame("Frame", nil, icon)
h:SetFrameLevel(4)
h:SetPoint("TOPLEFT",-3,3)
h:SetPoint("BOTTOMRIGHT",3,-3)
lib.gen_backdrop(h)
end
lib.createAuraWatch = function(self, unit)
if cfg.showAuraWatch then
local auras = {}
local spellIDs = {
DEATHKNIGHT = {
},
DRUID = {
33763, -- Lifebloom
8936, -- Regrowth
774, -- Rejuvenation
48438, -- Wild Growth
},
HUNTER = {
34477, -- Misdirection
},
MAGE = {
},
PALADIN = {
53563, -- Beacon of Light
25771, -- Forbearance
},
PRIEST = {
17, -- Power Word: Shield
139, -- Renew
33076, -- Prayer of Mending
6788, -- Weakened Soul
},
ROGUE = {
57934, -- Tricks of the Trade
},
SHAMAN = {
974, -- Earth Shield
61295, -- Riptide
},
WARLOCK = {
20707, -- Soulstone Resurrection
},
WARRIOR = {
},
}

auras.onlyShowPresent = true
auras.anyUnit = true
auras.PostCreateIcon = AWPostCreateIcon
-- Set any other AuraWatch settings
auras.icons = {}

for i, sid in pairs(spellIDs[playerClass]) do
local icon = CreateFrame("Frame", nil, self)
icon.spellID = sid
-- set the dimensions and positions
icon:SetWidth(14)
icon:SetHeight(14)
icon:SetFrameLevel(5)
icon:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", 12 * i, 5)

auras.icons[sid] = icon
end
self.AuraWatch = auras
end
end

-- oUF_Experience
lib.gen_exp = function(self)
if(IsAddOnLoaded('oUF_Experience')) and UnitLevel("player") ~= MAX_PLAYER_LEVEL_TABLE[GetExpansionLevel()] then
local Experience = CreateFrame("StatusBar", nil, self)
Experience:SetPoint("TOPLEFT", self.Health, "BOTTOMLEFT", 8, 8)
Experience:SetStatusBarTexture(cfg.xpbar_texture)
Experience:SetStatusBarColor(0.8, 0.1, 1, 0.8)
Experience:SetHeight(16)
Experience:SetWidth(self.Health:GetWidth())
Experience:SetFrameLevel(1)
fixStatusbar(Experience)

local h = CreateFrame("Frame", nil, Experience)
h:SetFrameLevel(0)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_expback(h)

local Rested = CreateFrame("StatusBar", nil, Experience)
Rested:SetStatusBarTexture(cfg.xpbar_texture)
Rested:SetStatusBarColor(0, 0.4, 1, 0.8)
Rested:SetAllPoints(Experience)
fixStatusbar(Rested)

local bg = Rested:CreateTexture(nil, 'BACKGROUND')
bg:SetAllPoints(Experience)
bg:SetTexture(cfg.xpbar_texture)
bg:SetVertexColor(0.05, 0.05, 0.05, 0.4)

local textframe = CreateFrame("Frame", nil, Experience)
textframe:SetFrameLevel(10)
textframe:SetAllPoints(Experience)
textframe:SetAlpha(0)

local exptext = lib.gen_fontstring(textframe, cfg.font, 12, "THINOUTLINE")
exptext:SetPoint("CENTER", textframe, "BOTTOM", 0,0)
self:Tag(exptext, '[sqw:curxp] / [sqw:maxxp] - [sqw:perxp]%')

-- Mouseover!
textframe:HookScript('OnEnter', function(self) self:SetAlpha(1) end)
textframe:HookScript('OnLeave', function(self) self:SetAlpha(0) end)
textframe:EnableMouse(true)

self.Experience = Experience
self.Experience.Rested = Rested

end
end

-- oUF_Reputation
lib.gen_rep = function(self)
if(IsAddOnLoaded("oUF_Reputation")) and UnitLevel("player") == MAX_PLAYER_LEVEL_TABLE[GetExpansionLevel()] then
local Reputation = CreateFrame("StatusBar", nil, self)
Reputation:SetPoint("TOPLEFT", self.Health, "BOTTOMLEFT", 8, 8)
Reputation:SetStatusBarTexture(cfg.xpbar_texture)
Reputation:SetStatusBarColor(0.8, 0.1, 1, 0.8)
Reputation:SetHeight(16)
Reputation:SetWidth(self.Health:GetWidth())
Reputation:SetFrameLevel(1)
fixStatusbar(Reputation)

-- Color the bar by current standing
Reputation.colorStanding = true

local h = CreateFrame("Frame", nil, Reputation)
h:SetFrameLevel(0)
h:SetPoint("TOPLEFT",-5,5)
h:SetPoint("BOTTOMRIGHT",5,-5)
lib.gen_expback(h)

local textframe = CreateFrame("Frame", nil, Reputation)
textframe:SetFrameLevel(10)
textframe:SetAllPoints(Reputation)
textframe:SetAlpha(0)

local reptext = lib.gen_fontstring(textframe, cfg.font, 10, "THINOUTLINE")
reptext:SetPoint("CENTER", textframe, "BOTTOM", 0,0)
self:Tag(reptext, '[reputation] - [currep] / [maxrep]')

-- Mouseover!
textframe:HookScript('OnEnter', function(self) self:SetAlpha(1) end)
textframe:HookScript('OnLeave', function(self) self:SetAlpha(0) end)
textframe:EnableMouse(true)

self.Reputation = Reputation
end
end

-- oUF_CombatIndicatorFeedback
lib.gen_combat_feedback = function(self)
if IsAddOnLoaded("oUF_CombatFeedback") and cfg.CombatFeedback then
local h = CreateFrame("Frame", nil, self.Health)
h:SetAllPoints(self.Health)
h:SetFrameLevel(30)
local cfbt = lib.gen_fontstring(h, cfg.font, 18, "THINOUTLINE")
cfbt:SetPoint("CENTER", self.Health, "BOTTOM", 0, -1)
cfbt.maxAlpha = 0.75
cfbt.ignoreEnergize = true
self.CombatIndicatorFeedbackText = cfbt
end
end

-- oUF_FloatingCombatIndicatorFeedback
lib.gen_floating_combat_feedback = function(self)
if IsAddOnLoaded("oUF_FloatingCombatFeedback") and cfg.FloatingCombatFeedback then
self.FloatingCombatFeedback = CreateFrame("Frame", nil, self.Health)
self.FloatingCombatFeedback:SetFrameLevel(30)
self.FloatingCombatFeedback:SetPoint("CENTER", self.Health, "BOTTOM", 0, -1)
for i = 1, 6 do
self.FloatingCombatFeedback[i] = lib.gen_fontstring(self.FloatingCombatFeedback, cfg.font, 18, "THINOUTLINE")
end
self.FloatingCombatFeedback.ignoreEnergize = true
if cfg.FountainMode then
self.FloatingCombatFeedback.Mode = "Fountain"
else
self.FloatingCombatFeedback.Mode = "Standard"
end
end
end

-- oUF_WeaponEnchant (temporary weapon enchant icon)
lib.gen_WeaponEnchant = function(self)
if IsAddOnLoaded("oUF_WeaponEnchant") then
self.Enchant = CreateFrame("Frame", nil, self)
self.Enchant:SetSize(24, 52)
self.Enchant:SetPoint("TOPLEFT", self.Health, "TOPRIGHT", 10, 14)
self.Enchant.size = 20
self.Enchant.spacing = 6
self.Enchant.initialAnchor = "LEFT"
self.Enchant["growth-y"] = "DOWN"
self.Enchant:SetFrameLevel(10)

self.PostCreateEnchantIcon = myPostCreateIcon
self.PostUpdateEnchantIcon = myPostUpdateIcon
end
end

--hand the lib to the namespace for further usage
ns.lib = lib